#
# ATBugTracking.py
#
# Clase para administrar avisos de error y su resolución
#
# Alejandro Álvarez Ayllón - 19/12/2006
#

from Products.Archetypes.public import BaseSchema, Schema
from Products.Archetypes.public import TextField
from Products.Archetypes.public import RichWidget
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import BaseContent, registerType
from config import PROJECTNAME, I18N_DOMAIN

from AccessControl import ClassSecurityInfo, getSecurityManager

from ZODB.PersistentList import PersistentList

from Bug import Bug

# Campos
schema = BaseSchema +  Schema((
    TextField('text',
              searchable = 1,
              required   = 0,
              widget     = RichWidget(label       = 'Text',
                                      label_msgid = 'text',
                                      i18n_domain = I18N_DOMAIN,
                                      ),
              ),
    ),
                              )

schema['description'].schemata = 'default'
schema['title'].mutator = 'setTitle' # Método a invocar cuando cambie el campo 'title'

# Campo
class ATBugTracking(BaseContent):
    """BugTracking"""

    schema = schema

    content_icon    = 'bug.png'
    meta_type       = 'ATBugTracking'
    portal_type     = 'BugTracking'
    archetype_name  = 'BugTracking'
    typeDescription = 'BugTracking'

    security = ClassSecurityInfo()

    actions = ({
	'id'         : 'view',
	'name'       : 'View',
	'action'     : 'string:${object_url}/bugtracking_view',
	'permissions': (CMFCorePermissions.View,)
	},)

    pagesize = 15

    # Inicialización
    def __init__(self, oid, **kwargs):
        """
        """
        BaseContent.__init__(self, oid, **kwargs)
        # Lista de bugs registrados
        self.bugsList = PersistentList()    

    # Cambia el ID de forma que sea adecuado al título
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setTitle')
    def setTitle(self, value):
        """
        Descripción: Cambia el ID del objeto de acuerdo al nuevo título.
        Parámetros:  value = Nuevo título.
        Devuelve:    Nada.
        """
        plone_tool = getToolByName(self, 'plone_utils')
        self.setId(plone_tool.normalizeString(value))
        self.title = value

    # Devuelve la lista de bugs empezando por el final (los más recientes)
    security.declareProtected(CMFCorePermissions.View, 'getBugs')
    def getBugs(self, page = 0):
        """
        Descripción: Devuelve la lista de bugs en orden inverso
        Parámetros:  Ninguno.
        Devuelve:    La lista de bugs en orden inverso.
        """

        tmp = list(self.bugsList)
        tmp.reverse()

        page = int(page)
        
        return tmp[page * self.pagesize:(page + 1) * self.pagesize]

    # Número de páginas
    security.declareProtected(CMFCorePermissions.View, 'getPageCount')
    def getPageCount(self):
        """
        Descripción: Devuelve el número de páginas.
        Parámetros:  Ninguno.
        Devuelve:    Un entero.
        """
        import math

        return math.ceil(len(self.bugsList) / float(self.pagesize))

    # Maneja el formulario de un nuevo bug
    security.declareProtected(CMFCorePermissions.View, 'newReport')
    def newReport(self, REQUEST):
        """
        Descripción: Maneja el formulario de nuevo bug.
        Parámetros:  REQUEST = Objeto request.
        Devuelve:    Nada. Redirige a la página principal del BugTracking.
        """

        form = REQUEST.form
        
        # Usuario
        uid = getSecurityManager().getUser().getId()
        if uid is None:
            uid = 'Anonymous'

        # Validamos datos
        if form['title'] == '':
            return 'A title is required'
        elif form['description'] == '':
            return 'A description is required'
        
        # Nuevo bug
        bug = Bug(self, len(self.bugsList), uid, form['title'], form['description'],
                  int(form['priority']), form['attachment'])

        # Añadimos a la lista
        self.bugsList.append(bug)

        # Redirigimos
        REQUEST.response.redirect(self.absolute_url())

    # Devuelve un bug en concreto (el indicado con la clave ID)
    security.declareProtected(CMFCorePermissions.View, 'getBug')
    def getBug(self, REQUEST):
        """
        Descripción: Devuelve el objeto Bug pedido o None si no existe.
        Parámetros:  REQUEST = El objeto request.
        Devuelve:    Un objeto Bug o None.
        """

        if 'id' in REQUEST.keys():
            bid = int(REQUEST['id'])
    
            if bid < len(self.bugsList):
                return self.bugsList[bid]
            else:
                return None
        else:
            return None

    # Indica si el usuario puede administrar los bugs
    security.declareProtected(CMFCorePermissions.View, 'isAdmin')
    def isAdmin(self):
        """
        Descripción: Indica si el usuario actual tiene permisos de administración.
        Parámetros:  Ninguno.
        Devuelve:    Un booleano.
        """

        uid = getSecurityManager().getUser().getId()

        if (uid in self.__ac_local_roles__.keys() and \
           'Manager' in self.__ac_local_roles__[uid]) or \
           uid == self.Creator():
            return True
        else:
            return 

# Registrar el tipo
registerType(ATBugTracking, PROJECTNAME)
