#
# ForjaTool.py
#
# Empleado para configurar el funcionamiento de los tipos de
# contenido de Forja
#
# Alejandro Álvarez Ayllón - 13/04/2006
#

from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, SimpleItemWithProperties
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, DTMLFile

from config import TOPIC_LIST, dtmlDir, LICENSE_URL_DICT

import os
from popen2 import Popen3

# Clase
class ForjaTool(UniqueObject, SimpleItemWithProperties):
    """
    Clase ForjaTool.
    Herramienta para la configuración de los contenidos.
    """

    __implements__ = (SimpleItemWithProperties.__implements__)

    id        = 'forja_tool'
    meta_type = 'forja_tool'

    manage_options= SimpleItemWithProperties.manage_options + \
                   ( { 'label' : 'Update'
                     , 'action' : 'manage_UpdateRepository'
                     },
                   )
    manage_UpdateRepository = DTMLFile('manage_UpdateRepository', dtmlDir)

    # Seguridad
    security  = ClassSecurityInfo()

    security.declareProtected(ManagePortal, 'manage_editProperties')
    security.declareProtected(ManagePortal, 'manage_changeProperties')
    security.declareProtected(ManagePortal, 'manage_propertiesForm')

    # Propiedades
    _properties = (
        {'id': 'viewcvs_script', 'type': 'string', 'mode': 'w',
         'label': 'ViewCVS script path',},
        {'id': 'repository_path', 'type': 'string', 'mode': 'w',
         'label': 'Repository path'},
        {'id': 'repository_passwd', 'type': 'string', 'mode': 'w',
         'label': 'Repository users file'},
        {'id': 'repository_authz', 'type': 'string', 'mode': 'w',
         'label': 'Repository Authz File'},
        {'id': 'repository_chmod', 'type': 'string', 'mode': 'w',
         'label': 'Repository creation permissions'},
        {'id': 'repository_host', 'type': 'string', 'mode': 'w',
         'label': 'Repository host',},        
        {'id': 'repository_anon', 'type': 'string', 'mode': 'w',
         'label': 'Unauthenticated permissions'},
        {'id': 'repository_auth', 'type': 'string', 'mode': 'w',
         'label': 'Authenticated permissions'},
        {'id': 'enable_repository', 'type': 'boolean', 'mode': 'w',
         'label': 'Enable repository'},
        {'id': 'enable_maillists', 'type': 'boolean', 'mode': 'w',
         'label': 'Enable mail lists'},
        {'id': 'generate_pdf_forms', 'type': 'boolean', 'mode': 'w',
         'label': 'Generate PDF Forms'},
        {'id': 'generate_html_forms', 'type': 'boolean', 'mode': 'w',
         'label': 'Generate HTML Forms'},
        )

    # Valores por defecto de las propiedades
    viewcvs_script    = '/usr/lib/cgi-bin/viewcvs.cgi'
    repository_path   = '/var/svn'
    repository_passwd = '/var/svn/passwd'
    repository_authz  = '/var/svn/authz'
    repository_chmod= '775'
    repository_host = 'svn://example.com:7000'
    repository_anon = 'read'
    repository_auth = 'write'
    enable_repository   = True
    enable_maillists    = True
    generate_pdf_forms  = True
    generate_html_forms = True

    # Inicialización del árbol
    def categoryTree(self):
        """
        Descripción: Genera el árbol de categorías.
        Parámetros:  Ninguno.
        Devuelve:    El árbol de categorías.
        """
        tree = {}
        # Generar el árbol de navegación
        for i in TOPIC_LIST:
            spl = i.split(' :: ')
            lista = tree
            for j in spl:
                if not lista.has_key(j):
                    lista[j] = {}
                lista = lista[j]
                
        return tree

    # Devuelve la sublista de un tema
    def getTopicList(self, topic):
        """
        Descripción: Devuelve la sublista correspondiente a topic.
        Parámetros:  topic: Nombre del tema del que se quiere obtener la sublista.
        Devuelve:    La sublista de topic.
        """
        spl = topic.split('/')
        lista = self.categoryTree()
        
        for i in spl:
            if i:
                lista = lista[i]

        return lista.keys()

    # Devuelve la URL donde se encuentra la licencia pedida
    def licenseUrl(self, license):
        """
        Descripción: Devuelve la dirección donde se encuentra la licencia.
        Parámetros:  license: Nombre de la licencia.
        Devuelve:    La dirección de la licencia, o '#' si no se encuentra.
        """

        if license in LICENSE_URL_DICT.keys():
            return LICENSE_URL_DICT[license]
        else:
            return '#'

    # Devuelve el contenido generado por ViewCVS
    def ViewCVS(self, unix_name, physical_path):
        """
        Descripción: Permite mostrar la página generada por el script ViewCVS
                     dentro de una plantilla.
                     Si el tipo no es texto plano o html, se escribe el contenido
                     directamente en la salida
        Parámetros:  unix_name     = Nombre del proyecto.
                     physical_path = La URL del objeto que llama a esta función.
                     (Para poder extraer la ruta del repositorio)
        Devuelve:    El contenido devuelto por ViewCVS si es texto plano o html,
                     si no lo es, lo escribe diréctamente por la salida.
        """
        response = self.REQUEST.response
        query_string = self.REQUEST.QUERY_STRING
        path_info = self.REQUEST.PATH_INFO
        VIEWCVS_CGI = self.viewcvs_script

        # Establecemos las variables de entorno
        # para que el script funcione adecuadamente
        obj_path = '/'.join(physical_path) + '/project_repository'
        path = path_info.replace(obj_path, '')

        if path == '':
            path = '/%s/' % unix_name

        os.environ['PATH_INFO']    = path 
        os.environ['SCRIPT_NAME']  = obj_path
        os.environ['QUERY_STRING'] = query_string

        # Comprobamos que existe el script
        if not os.path.exists(VIEWCVS_CGI):
            raise IOError('%s doesn\'t exist' % VIEWCVS_CGI)

        # Ejecutamos el script
        pipe = os.popen(VIEWCVS_CGI, 'r')
        # Inicializamos las variables
        content = ''
        is_text = 0

        # Leemos la cabecera        
        string = pipe.readline()
        
        while string != '\r\n':
            key_val = string.split(':')
            
            if key_val[0] == 'Content-Type': # Nos indica el mime-type del archivo
                key_val[1] = key_val[1].strip()
                if key_val[1] == 'text/html' or key_val[1] == 'text/xhtml': # El HTML lo embebemos
                    is_text = 1
                else:
                    response.setHeader('Content-Type', key_val[1]) # Cambiamos el tipo
                    
            string = pipe.readline()
            
        # Mandamos los datos
        if is_text:
            for string in pipe:
                content += string      # Metemos todo el contenido en una cadena
        else:
            for string in pipe:
                response.write(string) # Escribimos directamente si no es html

        pipe.close()
        
        # Eliminamos cabecera y pie HTML
        if content != '':
            lcontent = content.lower()
            i = lcontent.find('<body')
            start = lcontent.find('>', i) + 1
            end = lcontent.find('</body>')

            if start != -1 and end != -1:
                return content[start:end]
            else:
                return content
        else:     
            return None
 
    # Cambia el nombre del repositorio en el disco
    def moveOrCreateRepository(self, old, new, owner):
        """
        Descripción: Cambia el nombre del repositorio en el disco o lo crea si no existe.
        Parámetros:  old   = El antiguo nombre. Si es equivalente a falso
                             (cadena vacía, 0, None, False) crea el repositorio.
                             Si no lo es y el repositorio existe, le cambia el nombre.
                             Si no lo es y el repositorio no existe, lo crea.
                     new   = El nuevo nombre.
                     owner = El ID del usuario propietario.
        Devuelve:    Nada.
        """
        # Generamos la ruta del nuevo
        new_repos  = '%s/%s' % (self.repository_path, new)

        # Comprobamos que el ID no está ya cogido
        if os.access(new_repos, os.F_OK):
            raise IOError('Unix name already exists')

        #Cambiamos si está definido old y existe
        if old and os.access('%s/%s' % (self.repository_path, old), os.F_OK):
            repository = '%s/%s' % (self.repository_path, old)
            os.rename(repository, new_repos)
            # Eliminar del archivo authz
            authz = open(self.repository_authz, 'r')
            new_authz = open('%s_n' % self.repository_authz, 'w')

            for line in authz:
                if line == '[%s:/]\n' % old:
                    line = '[%s:/]\n' % new
                new_authz.write(line)

            new_authz.close()
            authz.close()

            os.remove(self.repository_authz)
            os.rename('%s_n' % self.repository_authz, self.repository_authz)
        else:
            # Creamos si no está definido o no existe.
            # Ejecutamos con una tubería para poder obtener la salida de error si hay alguno
            svnadmin = Popen3('svnadmin create %s' % new_repos, True)
            
            if svnadmin.wait() != 0:
                raise IOError('The repository cannot be created (%s):\n%s' % (new_repos, svnadmin.childerr.read()))
            

            # Añadimos el proyecto con sus permisos y sus cosas a authz
            authz = open(self.repository_authz, 'a')
            
            authz.write('[%s:/]\n' % new)
            authz.write('%s = %s\n\n' % (owner, self.repository_auth))

            authz.close();

        # Cambiamos los permisos
        os.chmod(new_repos, int(self.repository_chmod, 8))


    # Cambia la contraseña de un usuario en el repositorio
    def editRepositoryMember(self, user, password):
        """
        Descripción: Cambia la contraseña de un usuario en el repositorio.
        Parámetros:  user     = El ID del usuario.
                     password = La nueva contraseña.
        Devuelve:    Nada.
        """

        if os.access(self.repository_passwd, os.F_OK):
            # Actualizar
            htpasswd = Popen3('htpasswd -mb "%s" "%s" "%s"' % (self.repository_passwd, user, password), True)
        else:
            # Crear y actualizar
            htpasswd = Popen3('htpasswd -cmb "%s" "%s" "%s"' % (self.repository_passwd, user, password), True)

        if htpasswd.wait() != 0:
            raise IOError('The passwd file cannot be modified:\n%s' % htpasswd.childerr.read())

        return

    # Elimina los usuarios indicados
    def deleteRepositoryMembers(self, ids):
        """
        Descripción: Elimina los usuarios indicados de todos los proyectos.
        Parámetros:  ids = La lista de los IDs de los usuarios que queremos borrar.
        Devuelve:    Nada.
        """
        # Obtenemos la lista de proyectos
        catalog = getToolByName(self, 'portal_catalog')
        result  = catalog.searchResults(Type = 'Project')

        # Actualizamos los proyectos que tengan alguno de estos usuarios
        for i in result:
            obj = i.getObject()
            obj.removeUsers(ids)
            

    # Actualizar las contraseñas y permisos de TODOS los usuarios
    def updateRepositoryMembers(self, REQUEST = None):
        """
        Descripción: Actualiza las contraseñas y permisos de los usuarios de TODOS los proyectos.
        Parámetros:  REQUEST = Opcional. Se usa para poder volver al panel de administración.
        """
        membership_tool = getToolByName(self, 'portal_membership')

        # Contraseñas
        for usr in membership_tool.getUsersList():
            self.editRepositoryMember(usr, membership_tool.getUserPassword(usr))

        # Permisos en el repositorio (mejor crearlo de 0, es más rápido)
        catalog = getToolByName(self, 'portal_catalog')
        result  = catalog.searchResults(Type = 'Project')
        authz   = open(self.repository_authz, 'w')

        authz.write('[/]\n');
        authz.write('* = %s\n\n' % self.repository_anon)

        for i in result:
            obj = i.getObject();
            authz.write('[%s:/]\n' % obj.unix_name)
            authz.write('%s = %s\n' % (obj.Creator(), self.repository_auth))
            for usr in obj.administrators:
                authz.write('%s = %s\n' % (usr, self.repository_auth))
            for usr in obj.members:
                authz.write('%s = %s\n' % (usr, self.repository_auth))

        authz.close()
        return

    # Actualiza el archivo con los permisos de un proyecto
    def updateProjectRepositoryMembers(self, prname, owner, admin, members):
        """
        Descripción: Actualiza el archivo con los permisos de un proyecto.
        Parámetros:  prname  = Nombre del proyecto.
                     owner   = ID del propietario.
                     admin   = Lista con los IDs de los administradores.
                     members = Lista con los IDs de los miembros.
        Devuelve:    Nada.
        """

        nauthz = '%s_n' % self.repository_authz

        authz     = open (self.repository_authz, 'r')
        new_authz = open(nauthz, 'w')

        for line in authz:
            new_authz.write(line)
            if line == '[%s:/]\n' % prname:
                # Escribir usuarios
                new_authz.write('%s = %s\n', owner, self.repository_auth)
                for usr in admin:
                    new_authz.write('%s = %s\n', usr, self.repository_auth)
                for usr in members:
                    new_authz.write('%s = %s\n', usr, self.repository_auth)

                # Seguimos leyendo hasta el siguiente proyecto
                line = authz.readline()
                while line[0] != '[':
                    line = authz.readline()
                # Siguiente proyecto, reanudamos el bucle
                new_authz.write(line)

        new_authz.close()
        authz.close()

        os.remove(self.repository_authz)
        os.rename(nauthz, self.repository_authz)

        return

# Inicializar
InitializeClass(ForjaTool)
