#
# MembershipTool.py
#
# Sustituye a membership_tool de Plone para registrar a los usuarios también
# en el repositorio. 
#
# Alejandro Álvarez Ayllón - 07/05/2006
#

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.MembershipTool import MembershipTool as BaseTool
from Products.CMFCore import CMFCorePermissions
from Globals import InitializeClass

from AccessControl import ClassSecurityInfo, getSecurityManager

from ZODB.PersistentMapping import PersistentMapping

class MembershipTool(BaseTool):
    """
    Clase MembershipTool
    Control de los usuarios. Hereda de MembershipTool de Plone.
    Necesario para actualizar también los usuarios de los repositorios.
    """

    meta_type = 'membership_tool'

    security = ClassSecurityInfo()

    __implements__ = BaseTool.__implements__

    def __init__(self):
        # Lista de los usuarios y las contraseñas. Se mantiene aquí porque en
        # versiones modernas de Zope no se puede acceder a las contraseñas, y
        # las necesitamos para mantener el repositorio.
        self._users_passwords = PersistentMapping()

        BaseTool.__init__(self)
    

    # Cambiar la contraseña
    security.declareProtected(CMFCorePermissions.SetOwnPassword, 'setPassword')
    def setPassword(self, password, domains = None):
        """
        Descripción: Cambiar la contraseña. Sobrecarga el método de la clase base.
        Parámetros:  password = La nueva contraseña.
                     domains  = Necesario para mantener la compatibilidad.
        Devuelve:    Lo mismo que el método heredado.
        """

        uid = getSecurityManager().getUser().getId()

        self._users_passwords[uid] = password

        forja_tool = getToolByName(self, 'forja_tool')
        if forja_tool.enable_repository:
            forja_tool.editRepositoryMember(uid, password)

        return BaseTool.setPassword(self, password, domains)

    # Crea un usuario
    security.declarePrivate('addMember')
    def addMember(self, id, password, roles, domains, properties = None):
        """
        Descripción: Añade un nuevo miembro. Sobrecarga el método de la clase base.
        Parámetros: id, password, roles, domains, properties.
        Devuelve:   Lo mismo que el método sobrecargado.
        """

        self._users_passwords[id] = password;
        
        return BaseTool.addMember(self, id, password, roles, domains, properties)
        

    # Borra un usuario
    security.declareProtected(CMFCorePermissions.ManageUsers, 'deleteMembers')
    def deleteMembers(self, member_ids, delete_memberareas = 1,
                      delete_localroles = 1):
        """
        Descripción: Borra un usuario. Sobrecarga el método de la clase base.
        Parámetros:  members_ids        = Una lista de los IDs de los usuarios que se quieren borrar.
                     delete_memberareas = ¿Borrar las áreas de usuario?
                     delete_localroles  = ¿Borrar los roles asignados?
        Devuelve:    Lo mismo que el método heredado.
        """

        forja_tool = getToolByName(self, 'forja_tool')
        if forja_tool.enable_repository:
            forja_tool.deleteRepositoryMembers(member_ids)

        for i in member_ids:
            del self._users_passwords[i]
            
        return BaseTool.deleteMembers(self, member_ids, delete_memberareas,
                                      delete_localroles)

    # Devuelve la contraseña de un usuario
    def  getUserPassword(self, uid):
        """
        """

        if uid in self._users_passwords.keys():
            return self._users_passwords[uid]
        else:
            return uid

    # Devuelve la lista de usuarios
    def getUsersList(self):
        return self._users_passwords.keys()

    
InitializeClass(MembershipTool)
